Module SVlib
!===============================================================================
!    SVlib module contains subroutines useful for SismoVi vers. 2. data handling
!-------------------------------------------------------------------------------
implicit none

private
public :: SismoExport, SVDataTest
integer, parameter, public :: FINIS=-1, BEGIN=1

Contains
!===============================================================================
Subroutine SismoExport(operation, trace, array, title, xlabel, ylabel,    &
                       traces, tracestep, var_tracestep, tr_overlap,      &
                       samples, samplestep, var_samplestep, sl_overlap,   &
                       dorder, fformat, fname)
!-------------------------------------------------------------------------------
!
! SismoExport  routine:
!    
!      Writes  files in format readable by SismoVi data visualiser.
!      Operation flag  =  1 (BEGIN) / -1 (FINIS) 
!      Export MUST always start with a first call using BEGIN and end with 
!      a last call using FINIS.
!      SINGLE allows for writing single traces of variable length.
!      MULTI  allows for writing an array containing a number of "traces" having 
!      the same length "samples" with trace step "tracestep" and 
!      sample step "samplestep". Steps can be also variable.
!      "dorder" parameter allows for different ordering (not yet implemented)
!      "fformat" parameter allows for different file formats (not yet implemented)
!
! Author: G.Seriani, OGS, Trieste 7/10/2004 - 20/12/2005
!--------------------------------------------------------------------------------
implicit none

character(*), optional :: title, xlabel, ylabel, dorder, fformat, fname
integer,      optional :: operation, traces, tr_overlap, samples, sl_overlap
real,         optional :: trace(:), array(:,:)
real,         optional :: tracestep,  var_tracestep(:)
real,         optional :: samplestep, var_samplestep(:)
integer, parameter :: SINGLE=10, MULTI=11,  nchout = 20    
integer       :: opt
logical, save :: started=.FALSE.

opt=0;

if(present( trace ))      opt = SINGLE          ! error is raised if both "trace" 
if(present( array ))      opt = opt + MULTI     ! and "array" are present
if(present( operation ))  opt = operation


select case (opt)
  case (BEGIN)
       Call Prolog; started =.TRUE.
  case (SINGLE)
       if(.NOT.started) Call Prolog; started =.TRUE.
       write(nchout,*) "["
       write(nchout,"(G10.4,2H, )") trace
       write(nchout,*) "],"
  case (MULTI)
       if(.NOT.started) Call Prolog; started =.TRUE.
       write(nchout,"(G10.4,2H, )") array
  case (FINIS)
       Call Epilog
  case default
       STOP "Export error"
end select

Contains
   Subroutine Prolog
      OPEN(UNIT=nchout, FILE=fname, FORM='FORMATTED', STATUS='REPLACE')
      write(nchout,*) "{"
      if(present( title  ))  write(nchout,*) "'title': '", title ,"',"
      if(present( ylabel ))  write(nchout,*) "'trace_label':'", ylabel ,"',"
      if(present( xlabel ))  write(nchout,*) "'sample_label':'", xlabel ,"',"
      if(present( traces ))  write(nchout,*) "'traces':" , traces  ,","
      if(present( samples )) write(nchout,*) "'samples':", samples ,","
      if(present( var_tracestep )) then
          write(nchout,*) "'trace_interval':["
          write(nchout,"(G10.4,2H, )") var_tracestep
          write(nchout,*) "],"
      else
          if(present( tracestep )) then
                write(nchout,*) "'trace_interval':", tracestep ,","
          else
                write(nchout,*) "'trace_interval':", 1. ,","
          end if
      end if
      if(present( var_samplestep )) then
          write(nchout,*) "'sampling_interval':["
          write(nchout,"(G10.4,2H, )") var_samplestep
          write(nchout,*) "],"
      else
          if(present( samplestep  )) then 
                write(nchout,*) "'sampling_interval':" , samplestep  ,","
          else
                write(nchout,*) "'sampling_interval':" , 1. ,","
          end if
      end if
      if(present( tr_overlap )) then
!               write(nchout,*) "'trace_overlap':", tr_overlap ,","
                write(nchout,*) "'trace_overlap':", 0. ,","
      else
                write(nchout,*) "'trace_overlap':", 0. ,","
      end if
      if(present( sl_overlap )) then
!               write(nchout,*) "'sampling_overlap':", sl_overlap ,","
                write(nchout,*) "'sampling_overlap':", 0. ,","
      else
                write(nchout,*) "'sampling_overlap':", 0. ,","
      end if
      write(nchout,*) "'data_order':'sample', # sample or trace"
      write(nchout,*) "'data':["
   End Subroutine Prolog
   
   Subroutine Epilog
      write(nchout,*) "]"
      write(nchout,*) "}"
      CLOSE(nchout)
   End Subroutine Epilog

End Subroutine SismoExport   
 
!===============================================================================
Subroutine SVDataTest()
!-------------------------------------------------------------------------------
!
! SVDataTest      is a fortran example for writing data files in format readable
!                 in SismoVi by DataVi module.
!                 It uses SVlib module for importing relevant definitions and
!                 SismoViExport routine.
!                 The routine is called by the Program SVDTest below
!-------------------------------------------------------------------------------
!USE SVlib        ! MUST be used in external codes and programs
implicit none

real, allocatable :: matrix(:,:), var_tstep(:), var_sstep(:)
integer :: i,  j,  dimX, dimY, vtp = 5, vsp = 6
real    :: si, tj

dimX=31
dimY=21

allocate(matrix(dimX,dimY), var_tstep(vtp), var_sstep(vsp))

! Generate an array of real values at constant step ----------------------------

do j=1,dimY
    do i=1,dimX
        matrix(i,j)=cos(sqrt(1.0*(i-(dimX+1)/2)**2+(j-(dimY+1)/2)**2)*0.2*3.14159)
    end do
end do
 
! Data are written in the file as a single 2D array of values ----------------

Call SismoExport(BEGIN, fname='Test1.sismo',                               &
                   title='Test 1 data',   xlabel='Time',   ylabel='Space',   &
                   traces=dimY, tracestep=1.,  samples=dimX, samplestep=1. )

Call SismoExport( array=matrix ) 

Call SismoExport(FINIS)
 
! Data are written in the file as  multi vectors of values -------------------
  
Call SismoExport(BEGIN, fname='Test2.sismo',                               &
                   title='Test 2 data',   xlabel='Time',   ylabel='Space',   &
                   tracestep=1.,  samplestep=1. )
do j=1,dimY
   Call SismoExport( trace=matrix(1:dimX, j) )
end do

Call SismoExport(FINIS)


! Generate an array of real values at variable step ----------------------------

do j=1,vtp
   var_tstep(j) = ((j-1.)**2)/4.
end do
do i=1,vsp
   var_sstep(i) = ((i-1.)**2)/5.
end do
do j=1,dimY
    do i=1,dimX
        tj = 1.+var_sstep(1+mod(j-1,vtp-1))+(vtp-1)*((j-1)/(vtp-1))
        si = 1.+var_sstep(1+mod(i-1,vsp-1))+(vsp-1)*((i-1)/(vsp-1))
        print *, "  j ",j,"-",tj,"  i ",i,"-",si
        matrix(i,j)=cos(sqrt(1.0*(si-(dimX+1)/2)**2+(tj-(dimY+1)/2)**2)*0.2*3.14159)
    end do
end do
 
! Data are written in the file as a single 2D array of values ----------------

Call SismoExport(BEGIN, fname='Test3.sismo',                               &
                   title='Test 3 data  (variable steps)',                  & 
                   xlabel='Time', ylabel='Space',                          &
                   traces=dimY,  var_tracestep=var_tstep,                  &                     
                   samples=dimX, var_samplestep=var_sstep )

Call SismoExport( array=matrix ) 

Call SismoExport(FINIS)
!
End Subroutine SVDataTest
!===============================================================================
End Module SVlib    
! 
!*******************************************************************************        
!> Program SVDTest   ! Delete the comment mark !> in order to run it !!!!
!*******************************************************************************
!> USE SVlib
!> implicit none
!> Call SVDataTest()
!> End Program SVDTest