###### simovilib.py
###### contains advanced functions external to SismoVi

# import userlib

import numarray as N
import matplotlib as M

def MovingAverage(data, points=3):
    pabs=N.abs(points)
    psx=(pabs-1)/2+(pabs-1)%2*(-points<0)
    pdx=(pabs-1)/2+(pabs-1)%2*(points<0)
    (rows,cols)=data.shape
    b=N.zeros((rows,cols+pabs-1),N.Float)
    b[:,psx:cols+psx]=data[:,:]
    for k in range(psx):
        b[:,k]=data[:,0]
    for k in range(pdx):
        b[:,psx+cols+k]=data[:,cols-1]
    data[:,:]=b[:,pabs-1:cols+pabs-1].copy()
    for k in range(psx):
        data[:,:]+=b[:,k:cols+k]
    for k in range(pdx):
        data[:,:]+=b[:,psx+k:psx+cols+k]
    return data/float(pabs)


def ReSample(x, f, ns=1, axis=0):
    """ Sample a single/multiple sequence (series) of data f (1D/2D array),
        irregularly sampled at coordinates x (1D array). If f is a multiple
        sequence (2D) all components have the same coordinate vector x.
      - Linear interpolation and constant step increment are used.
      - ns  is the re-samplig factor, i.e. ns=1 yields the same I/O samples.
      - axis is the dimension axis, 0 for the first and 1 for the second axis.
      - (xi, fi)  are the new coordinate and value arrays that are returned 
        with the same shapes as (x, f).
    """    
    nx = len(x);        dim = len(N.shape(f))
    n0 = N.shape(f)[0];  
    if dim == 2:  n1 = N.shape(f)[1]
    ni = int(N.around(nx*ns))
    l1 = N.arange(ni);     x1 = x[0]
    dx = (x[-1] - x[0])/(ni - 1.)
    xi = N.fromfunction(lambda i: x1 + i*dx, (ni,) )
    k  = 0 
    for i in range(ni): 
       while k<nx and x[k] <= xi[i]:  k += 1 
       k -= 1 ;  l1[i] = k      #;print(i,"->",k)
    l1[-1] = nx - 2              
    l2  = l1+1
    fdi = (xi - x[l1])/(x[l2] - x[l1]) 
    if dim == 1 :
       fi = f[l1] + (f[l2] - f[l1])*fdi
    elif dim == 2 and axis == 0:
       fdi = N.reshape(fdi,(ni,1))
       fdi = N.take(fdi,N.zeros(n1),axis=1)
       f1  = N.take(f,l1); f2 = N.take(f,l2)  
       fi  = f1 + (f2 - f1)*fdi
    elif dim == 2 and axis == 1:
       fdi = N.reshape(fdi,(1,ni))
       fdi = N.take(fdi,N.zeros(n0),axis=0) 
       f1  = N.take(f,l1,axis=1); f2 = N.take(f,l2,axis=1) 
       fi  = f1 + (f2 - f1)*fdi
    return xi, fi
  

def ReSample2D(x0, x1, f, ns0=1, ns1=1):
    """ Sample a 2D array (series) of data f irregularly sampled at 
        coordinates x0, x1 (1D arrays). Along each direction (0 or 1) the 
        components of the multiple series have the same coordinate vectors
        either x0 or x1..
      - Linear interpolation and constant step increment are used.
      - ns0,ns1  are re-samplig factors, i.e. ns0=1 yields the same I/O samples.
      - (x0i, x1i fi)  are the new coordinate and value arrays that are returned 
        with the same shapes as (x0, x1, f).
    """    
    a=ReSample(x0,    f, ns0, axis=0)

    b=ReSample(x1, a[1], ns1, axis=1)

    return a[0], b[0], b[1]
    